"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Collections = require("typescript-collections");
var chain_state_1 = require("./chain-state");
var markov_chain_items_1 = require("./markov-chain-items");
var markov_terminal_items_1 = require("./markov-terminal-items");
var utils = require("./utils");
var MarkovChain = (function () {
    function MarkovChain(order) {
        this.items = new markov_chain_items_1.MarkovChainItems();
        this.terminals = new markov_terminal_items_1.MarkovTerminalItems();
        if (order < 0) {
            throw new RangeError("Order must not be less than 0.");
        }
        this.order = order;
    }
    MarkovChain.prototype.add = function (items) {
        this.addWithWeight(items, 1);
    };
    MarkovChain.prototype.addWithWeight = function (items, weight) {
        var _this = this;
        var previous = new Collections.Queue();
        items.forEach(function (item) {
            var key = chain_state_1.ChainState.fromQueue(previous);
            _this.addWithPrecedingChainAndWeight(key, item, weight);
            previous.enqueue(item);
            if (previous.size() > _this.order) {
                previous.dequeue();
            }
        });
        var terminalKey = chain_state_1.ChainState.fromQueue(previous);
        var terminalWeight = weight;
        if (this.terminals.containsKey(terminalKey)) {
            terminalWeight += this.terminals.getValue(terminalKey);
        }
        ;
        this.terminals.setValue(terminalKey, terminalWeight);
    };
    MarkovChain.prototype.addWithPrecedingChainAndWeight = function (state, next, weight) {
        var weights;
        if (!this.items.containsKey(state)) {
            weights = new Collections.Dictionary();
            this.items.setValue(state, weights);
        }
        else {
            weights = this.items.getValue(state);
        }
        var newWeight = weight;
        if (weights.containsKey(next)) {
            newWeight += weights.getValue(next);
        }
        weights.setValue(next, newWeight);
    };
    MarkovChain.prototype.walk = function () {
        return this.walkWithPrevious([]);
    };
    MarkovChain.prototype.walkWithPrevious = function (previous) {
        var retVal = new Array();
        var state = new Collections.Queue();
        previous.forEach(function (x) { return state.add(x); });
        var _loop_1 = function () {
            while (state.size() > this_1.order) {
                state.dequeue();
            }
            var key = chain_state_1.ChainState.fromQueue(state);
            if (!this_1.items.containsKey(key)) {
                return "break";
            }
            var weights = this_1.items.getValue(key);
            var terminalWeight = 0;
            if (this_1.terminals.containsKey(key)) {
                terminalWeight = this_1.terminals.getValue(key);
            }
            var total = 0;
            weights.forEach(function (k, v) { total += v; return true; });
            var value = utils.randomNumber(total + terminalWeight) + 1;
            if (value > total) {
                return "break";
            }
            var currentWeight = 0;
            weights.forEach(function (k, v) {
                currentWeight += v;
                if (currentWeight >= value) {
                    retVal.push(k);
                    state.enqueue(k);
                    return false;
                }
                return true;
            });
        };
        var this_1 = this;
        while (true) {
            var state_1 = _loop_1();
            if (state_1 === "break")
                break;
        }
        return retVal;
    };
    return MarkovChain;
}());
exports.MarkovChain = MarkovChain;
;
//# sourceMappingURL=markov-chain.js.map